package com.yonge.cooleshow.admin.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;

import com.yonge.cooleshow.biz.dal.support.PageUtil;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.page.PageInfo;
import com.yonge.toolset.utils.string.StringUtil;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import com.yonge.cooleshow.biz.dal.vo.CourseHomeworkVo;
import com.yonge.cooleshow.biz.dal.dto.search.CourseHomeworkSearch;
import com.yonge.cooleshow.biz.dal.entity.CourseHomework;
import com.yonge.cooleshow.biz.dal.service.CourseHomeworkService;

@RestController
@RequestMapping("/courseHomework")
@Api(value = "课程作业表", tags = "课程作业表")
public class CourseHomeworkController extends BaseController {

    @Autowired
    private CourseHomeworkService courseHomeworkService;

	/**
     * 查询单条
     */
    @GetMapping("/detail/{id}")
    @ApiOperation(value = "详情", notes = "传入id")
    public HttpResponseResult<CourseHomeworkVo> detail(@PathVariable("id") Long id) {
    	return succeed(courseHomeworkService.detail(id));
	}
    
    /**
     * 查询分页
     */
    @PostMapping("/page")
    @ApiOperation(value = "查询分页", notes = "传入courseHomeworkSearch")
    public HttpResponseResult<PageInfo<CourseHomeworkVo>> page(@RequestBody CourseHomeworkSearch query) {
		IPage<CourseHomeworkVo> pages = courseHomeworkService.selectPage(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(pages));
	}
    
    /**
	 * 新增
	 */
	@PostMapping("/save")
	@ApiOperation(value = "新增", notes = "传入courseHomework")
	public HttpResponseResult save(@Valid @RequestBody CourseHomework courseHomework) {
    	return status(courseHomeworkService.save(courseHomework));
	}
    
    /**
	 * 修改
	 */
	@PostMapping("/update")
	@ApiOperation(value = "修改", notes = "传入courseHomework")
	public HttpResponseResult update(@Valid @RequestBody CourseHomework courseHomework) {
        return status(courseHomeworkService.updateById(courseHomework));
	}
    
    /**
	 * 新增或修改
	 */
    @PostMapping("/submit")
    @ApiOperation(value = "新增或修改", notes = "传入courseHomework")
	public HttpResponseResult submit(@Valid @RequestBody CourseHomework courseHomework) {
        return status(courseHomeworkService.saveOrUpdate(courseHomework));
    }

 	/**
	 * 删除
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public HttpResponseResult remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        if (StringUtil.isEmpty(ids)) {
			return failed("参数不能为空");
		}
		return status(courseHomeworkService.removeByIds(StringUtil.toLongList(ids)));
	}
}