package com.yonge.cooleshow.admin.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;

import com.yonge.cooleshow.biz.dal.support.PageUtil;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.page.PageInfo;
import com.yonge.toolset.utils.string.StringUtil;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import com.yonge.cooleshow.biz.dal.vo.UserAccountRecordVo;
import com.yonge.cooleshow.biz.dal.dto.search.UserAccountRecordSearch;
import com.yonge.cooleshow.biz.dal.entity.UserAccountRecord;
import com.yonge.cooleshow.biz.dal.service.UserAccountRecordService;

@RestController
@RequestMapping("/userAccountRecord")
@Api(value = "用户账户流水表", tags = "用户账户流水表")
public class UserAccountRecordController extends BaseController {

    @Autowired
    private UserAccountRecordService userAccountRecordService;

	/**
     * 查询单条
     */
    @GetMapping("/detail/{id}")
    @ApiOperation(value = "详情", notes = "传入id")
    public HttpResponseResult<UserAccountRecordVo> detail(@PathVariable("id") Long id) {
    	return succeed(userAccountRecordService.detail(id));
	}
    
    /**
     * 查询分页
     */
    @PostMapping("/page")
    @ApiOperation(value = "查询分页", notes = "传入userAccountRecordSearch")
    public HttpResponseResult<PageInfo<UserAccountRecordVo>> page(@RequestBody UserAccountRecordSearch query) {
		IPage<UserAccountRecordVo> pages = userAccountRecordService.selectPage(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(pages));
	}
    
    /**
	 * 新增
	 */
	@PostMapping("/save")
	@ApiOperation(value = "新增", notes = "传入userAccountRecord")
	public HttpResponseResult save(@Valid @RequestBody UserAccountRecord userAccountRecord) {
    	return status(userAccountRecordService.save(userAccountRecord));
	}
    
    /**
	 * 修改
	 */
	@PostMapping("/update")
	@ApiOperation(value = "修改", notes = "传入userAccountRecord")
	public HttpResponseResult update(@Valid @RequestBody UserAccountRecord userAccountRecord) {
        return status(userAccountRecordService.updateById(userAccountRecord));
	}
    
    /**
	 * 新增或修改
	 */
    @PostMapping("/submit")
    @ApiOperation(value = "新增或修改", notes = "传入userAccountRecord")
	public HttpResponseResult submit(@Valid @RequestBody UserAccountRecord userAccountRecord) {
        return status(userAccountRecordService.saveOrUpdate(userAccountRecord));
    }

 	/**
	 * 删除
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public HttpResponseResult remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        if (StringUtil.isEmpty(ids)) {
			return failed("参数不能为空");
		}
		return status(userAccountRecordService.removeByIds(StringUtil.toLongList(ids)));
	}
}