package com.yonge.cooleshow.admin.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;

import com.yonge.cooleshow.biz.dal.support.PageUtil;
import com.yonge.cooleshow.biz.dal.vo.UserOrderVo;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.page.PageInfo;
import com.yonge.toolset.utils.string.StringUtil;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import com.yonge.cooleshow.biz.dal.dto.search.OrderSearch;
import com.yonge.cooleshow.biz.dal.entity.UserOrder;
import com.yonge.cooleshow.biz.dal.service.UserOrderService;

@RestController
@RequestMapping("/userOrder")
@Api(value = "平台订单表", tags = "平台订单表")
public class UserOrderController extends BaseController {

    @Autowired
    private UserOrderService userOrderService;

	/**
     * 查询单条
     */
    @GetMapping("/detail/{id}")
    @ApiOperation(value = "详情", notes = "传入id")
    public HttpResponseResult<UserOrderVo> detail(@PathVariable("id") Long id) {
    	return succeed(userOrderService.detail(id));
	}
    
    /**
     * 查询分页
     */
    @PostMapping("/page")
    @ApiOperation(value = "查询分页", notes = "传入orderSearch")
    public HttpResponseResult<PageInfo<UserOrderVo>> page(@RequestBody OrderSearch query) {
		IPage<UserOrderVo> pages = userOrderService.selectPage(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(pages));
	}
    
    /**
	 * 新增
	 */
	@PostMapping("/save")
	@ApiOperation(value = "新增", notes = "传入order")
	public HttpResponseResult save(@Valid @RequestBody UserOrder userOrder) {
    	return status(userOrderService.save(userOrder));
	}
    
    /**
	 * 修改
	 */
	@PostMapping("/update")
	@ApiOperation(value = "修改", notes = "传入order")
	public HttpResponseResult update(@Valid @RequestBody UserOrder userOrder) {
        return status(userOrderService.updateById(userOrder));
	}
    
    /**
	 * 新增或修改
	 */
    @PostMapping("/submit")
    @ApiOperation(value = "新增或修改", notes = "传入order")
	public HttpResponseResult submit(@Valid @RequestBody UserOrder userOrder) {
        return status(userOrderService.saveOrUpdate(userOrder));
    }

 	/**
	 * 删除
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public HttpResponseResult remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        if (StringUtil.isEmpty(ids)) {
			return failed("参数不能为空");
		}
		return status(userOrderService.removeByIds(StringUtil.toLongList(ids)));
	}
}