package com.yonge.cooleshow.biz.dal.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.yonge.cooleshow.biz.dal.dao.CourseGroupDao;
import com.yonge.cooleshow.biz.dal.dto.CheckLiveCourseTimeDto;
import com.yonge.cooleshow.biz.dal.dto.LiveCourseGroupDto;
import com.yonge.cooleshow.biz.dal.entity.CourseGroup;
import com.yonge.cooleshow.biz.dal.entity.CourseTimeEntity;
import com.yonge.cooleshow.biz.dal.vo.CourseGroupVo;
import com.yonge.cooleshow.biz.dal.vo.LiveCourseInfoVo;
import com.yonge.cooleshow.common.page.PageInfo;
import org.redisson.api.RMap;

import java.util.List;
import java.util.Map;

/**
 * 课程组表(CourseGroup)表服务接口
 *
 * @author hgw
 * @since 2022-03-18 15:29:10
 */
public interface CourseGroupService extends IService<CourseGroup> {

    CourseGroupDao getDao();

    /**
     * 查询课程组详情-直播课详情
     *
     * @param groupId 课程组id
     */
    LiveCourseInfoVo queryLiveCourseInfo(Long groupId);

    /**
     * 分页查询课程组列表
     *
     * @param param 传入参数
     *              <p> - teacherId    老师id
     *              <p> - groupStatus  课程组状态  ING(进行中)  NOT_SALE(未开售,未上架) APPLY(报名中,销售中) COMPLETE(已完成)
     *              <p> - row 条数
     *              <p> - page 页数
     */
    PageInfo<CourseGroupVo> queryPageLiveCourseGroup(Map<String, Object> param);

    void addLiveCourse(LiveCourseGroupDto dto);

    /**
     * 创建直播课程组时将课时写到缓存当作锁定的时间
     */
    List<CourseTimeEntity> lockCourseToCache(CheckLiveCourseTimeDto dto);

    /**
     * 创建直播课程组-解除锁定课程时间-删除写到缓存当作锁定的课时
     */
    void unlockCourseToCache(Long teacherId);

    /**
     * 获取老师锁定的直播课时数据的缓存
     *
     * @param teacherId 老师id
     * @return 缓存 key teacherId value List<CourseTimeEntity>
     */
    RMap<Long, List<CourseTimeEntity>> getLiveLockTimeCache(Long teacherId);

}

