package com.yonge.cooleshow.biz.dal.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.yonge.cooleshow.biz.dal.dao.CourseScheduleDao;
import com.yonge.cooleshow.biz.dal.dto.search.MyCourseSearch;
import com.yonge.cooleshow.biz.dal.entity.CourseCalendarEntity;
import com.yonge.cooleshow.biz.dal.entity.CourseSchedule;
import com.yonge.cooleshow.biz.dal.vo.MyCourseVo;
import com.yonge.cooleshow.biz.dal.vo.TeacherLiveCourseInfoVo;
import com.yonge.cooleshow.common.page.PageInfo;

import javax.validation.Valid;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

/**
 * 老师课程表(CourseSchedule)表服务接口
 *
 * @author hgw
 * @since 2022-03-18 15:29:11
 */
public interface CourseScheduleService extends IService<CourseSchedule> {

    CourseScheduleDao getDao();

    /**
     * 老师端-首页-我的课程-直播课
     *
     * @param param 传入参数
     *              <p> - classDate 年月
     *              <p> - row 条数
     *              <p> - page 页数
     *              <p> - status 课程状态 NOT_START未开始 ING进行中 COMPLETE已完成
     */
    PageInfo<TeacherLiveCourseInfoVo> queryTeacherLiveCourse(Map<String, Object> param);

    /**
     * 校验该学生大于当前时间并未开始的课程时间和传入时间段有没有交集
     *
     * @param studentId 学生id
     * @param startTime 新增课程 开始时间
     * @param endTime   新增课程 结束时间
     * @return true 被占用 false 没有被占用
     */
    boolean checkStudentCourseTime(Long studentId, Date startTime, Date endTime);

    /**
     * 校验老师这个时间段有没有被占用-校验课时
     *
     * @param teacherId 老师id
     * @param startTime 新增课程 开始时间
     * @param endTime   新增课程 结束时间
     * @return true 被占用 false 没有被占用
     */
    boolean checkTeacherCourseTime(Long teacherId, Date startTime, Date endTime);

    /**
     * 查询这个课程列表中时间是否有重叠
     * <p>案例：1
     * <p>已有9:00~ 10:00 课程
     * <p>新增课程 09:10~ 09:40
     * <p>返回 true
     *
     * <p>案例：2
     * <p>已有9:00~ 10:00 课程
     * <p>新增课程 08:45~ 09:40
     * <p>返回 true
     *
     * <p>案例：3
     * <p>已有9:00~ 10:00 课程
     * <p>新增课程 10:00~ 10:40
     * <p>返回 false
     *
     * @param list         时间列表
     * @param startTimeFun 获取现课程开始时间
     * @param endTimeFun   获取现课程结束时间
     * @param startTime    新增课程开始时间
     * @param endTime      新增课程结束时间
     * @return true 被占用 false 没有被占用
     */
    <T> boolean checkCourseTime(List<T> list, Function<T, Date> startTimeFun, Function<T, Date> endTimeFun,
                                Date startTime, Date endTime);

    /**
     * 修改-锁定课时
     *
     * @param id 课程id
     */
    void lockCourseTime(Long id);

    /**
     * 修改-解锁课时
     *
     * @param id 课程id
     */
    void unlockCourseTime(Long id);

    /**
     * 添加课时
     */
    void add(@Valid CourseSchedule course);


    /**
     * 创建直播课时选课时的日历
     * <P>主要查询该老师目前的课程时间
     * <P>根据传入的单课时时长将时间分片后来匹配数据库中课程时间，得到每日空余时间
     *
     * @param param 传入参数
     *              <p> - teacherId 老师id
     *              <p> - year 年
     *              <p> - month 月
     *              <p> - day 日
     *              <p> - singleCourseMinutes 单课时时长(包含休息时间)
     * @return 返回传入时间当月每日的剩余时间段
     */
    List<CourseCalendarEntity> createLiveCourseCalendar(Map<String, Object> param);

    /**
     * 生成陪练课日历-用于学生购买指定老师陪练课
     *
     * @param param 传入参数
     *              <p> - teacherId 老师id
     *              <p> - studentId 学生id
     *              <p> - year 年
     *              <p> - month 月
     *              <p> - day 日
     * @return 返回传入时间当月每日的剩余时间段
     */
    List<CourseCalendarEntity> createPracticeCourseCalender(Map<String, Object> param);

    IPage<MyCourseVo> queryTeacherPracticeCourse(IPage<MyCourseVo> page, MyCourseSearch search);
}

