package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.biz.dal.dao.CourseScheduleDao;
import com.yonge.cooleshow.biz.dal.dao.CourseScheduleRepliedDao;
import com.yonge.cooleshow.biz.dal.dao.CourseScheduleStudentPaymentDao;
import com.yonge.cooleshow.biz.dal.entity.CourseSchedule;
import com.yonge.cooleshow.biz.dal.entity.CourseScheduleReplied;
import com.yonge.cooleshow.biz.dal.entity.CourseScheduleStudentPayment;
import com.yonge.cooleshow.biz.dal.enums.CourseScheduleEnum;
import com.yonge.cooleshow.biz.dal.service.CourseRepliedService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @Author: cy
 * @Date: 2022/4/12
 */
@Service
public class CourseRepliedServiceImpl extends ServiceImpl<CourseScheduleRepliedDao, CourseScheduleReplied> implements CourseRepliedService {
    private final static Logger log = LoggerFactory.getLogger(CourseRepliedServiceImpl.class);

    @Autowired
    private CourseScheduleRepliedDao repliedDao;
    @Autowired
    private CourseScheduleDao scheduleDao;
    @Autowired
    private CourseScheduleStudentPaymentDao paymentDao;

    public CourseScheduleRepliedDao getDao() {
        return repliedDao;
    }

    /**
     * @Description: 首页-我的课程-课程详情-评价陪练课
     * @Author: cy
     * @Date: 2022/4/12
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void replied(CourseScheduleReplied replied, SysUser sysUser) {
        Long userId = sysUser.getId();
        Long scheduleId = replied.getCourseScheduleId();
        Long studentId = replied.getStudentId();

        CourseSchedule schedule = scheduleDao.selectById(scheduleId);
        if (schedule == null) {
            throw new RuntimeException("课程不存在");
        }
        if (schedule.getStatus().equals(CourseScheduleEnum.NOT_START.getCode())) {
            throw new RuntimeException("课程未开始无法评论");
        }

        //评论存在执行修改
        QueryWrapper<CourseScheduleReplied> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("student_id_", replied.getStudentId());
        queryWrapper.eq("course_schedule_id_", replied.getCourseScheduleId());
        queryWrapper.eq("course_group_id_", replied.getCourseGroupId());
        queryWrapper.eq("course_group_type_", CourseScheduleEnum.PRACTICE.getCode());
        CourseScheduleReplied scheduleReplied = repliedDao.selectOne(queryWrapper);
        if (scheduleReplied != null) {
            repliedDao.updateReplied(replied);
            return;
        }

        //当前用户为老师
        CourseSchedule courseSchedule = scheduleDao.selectById(scheduleId);
        if (courseSchedule != null) {
            Long teacherId = courseSchedule.getTeacherId();
            if (userId == teacherId) {
                repliedDao.insert(replied);
                return;
            }
        }

        //当前用户为学员
        QueryWrapper<CourseScheduleStudentPayment> wrapper = new QueryWrapper<>();
        wrapper.eq("user_id_", userId);
        wrapper.eq("course_id_", scheduleId);
        wrapper.eq("course_type_", CourseScheduleEnum.PRACTICE);
        CourseScheduleStudentPayment payment = paymentDao.selectOne(wrapper);
        if (payment == null || userId != studentId) {
            throw new RuntimeException("未购买该课无法评论");
        }
        repliedDao.insert(replied);
    }

    /**
     * @Description: 首页-我的课程-课程详情-查询评价
     * @Author: cy
     * @Date: 2022/4/12
     */
    @Override
    public CourseScheduleReplied selectReplied(CourseScheduleReplied replied) {
        QueryWrapper<CourseScheduleReplied> wrapper = new QueryWrapper<>();
        wrapper.eq("course_group_type_", CourseScheduleEnum.PRACTICE);
        wrapper.eq("student_id_", replied.getStudentId());
        wrapper.eq("course_schedule_id_", replied.getCourseScheduleId());
        wrapper.eq("course_group_id_", replied.getCourseGroupId());
        return repliedDao.selectOne(wrapper);
    }

}